@@ -410,8 +410,10 @@ def geometry_window(dataset, shapes, pad_x=0, pad_y=0, north_up=True,
         right, bottom = (right, bottom) * dataset.transform
 
     window = dataset.window(left, bottom, right, top)
-    window = window.round_offsets(op='floor', pixel_precision=pixel_precision)
-    window = window.round_shape(op='ceil', pixel_precision=pixel_precision)
+    window_floored = window.round_offsets(op='floor', pixel_precision=pixel_precision)
+    w = math.ceil(window.width + window.col_off - window_floored.col_off)
+    h = math.ceil(window.height + window.row_off - window_floored.row_off)
+    window = Window(window_floored.col_off, window_floored.row_off, w, h)
 
     # Make sure that window overlaps raster
     raster_window = Window(0, 0, dataset.width, dataset.height)
